@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -59,6 +59,7 @@ public class PropertyUtilTest {
         private String throwsWhenSet;
         private String requiresTwo;
         private String notReadable;
+        private int intOption;
 
         private int numberValue;
         private boolean booleanValue;
@@ -160,6 +161,18 @@ public class PropertyUtilTest {
         public void setStringArray(String[] stringArray) {
             this.stringArray = stringArray;
         }
+
+        public void setIntOption(int value) {
+            this.intOption = value;
+        }
+
+        public void setIntOption(String value) {
+            this.intOption = Integer.parseInt(value);
+        }
+
+        public int getIntOption() {
+            return intOption;
+        }
     }
 
     @Test
@@ -167,6 +180,8 @@ public class PropertyUtilTest {
         new PropertyUtil();
     }
 
+    //----- replaceQuery -----------------------------------------------------//
+
     @Test
     public void testReplaceQueryUsingMap() throws URISyntaxException {
         URI original = new URI("http://www.example.com?option=true");
@@ -187,6 +202,108 @@ public class PropertyUtilTest {
         assertEquals("param=replaced", updated.getQuery());
     }
 
+    @Test
+    public void testReplaceQueryPreservesFragment() throws URISyntaxException {
+        URI original = new URI("http://www.example.com?option=true#fragment");
+        URI updated = PropertyUtil.replaceQuery(original, "param=replaced");
+
+        assertEquals("param=replaced", updated.getQuery());
+        assertEquals("fragment", updated.getFragment());
+    }
+
+    @Test
+    public void testReplaceQueryPreservesFragmentWhenNoQueryPresent() throws URISyntaxException {
+        URI original = new URI("http://www.example.com#fragment");
+        URI updated = PropertyUtil.replaceQuery(original, "param=replaced");
+
+        assertEquals("param=replaced", updated.getQuery());
+        assertEquals("fragment", updated.getFragment());
+    }
+
+    @Test
+    public void testReplaceQueryWithStringDoesNotReencode() throws URISyntaxException {
+        URI original = new URI("http://www.example.com?option=X");
+
+        final String encodedValue = "%25Ca%2BHn%2Fav";
+        final String decodedValue = "%Ca+Hn/av";
+
+        final String encodedKey = "user%2Bname";
+        final String decodedKey = "user+name";
+
+        String newQuery = encodedKey + "=" + encodedValue;
+
+        URI updated = PropertyUtil.replaceQuery(original, newQuery);
+
+        assertEquals(encodedKey + "=" + encodedValue, updated.getRawQuery());
+        assertEquals(decodedKey + "=" + decodedValue, updated.getQuery());
+    }
+
+    @Test
+    public void testReplaceQueryUsingMapEncodesParameters() throws URISyntaxException {
+        URI original = new URI("http://www.example.com?option=X");
+
+        final String encodedValue = "%25Ca%2BHn%2Fav";
+        final String decodedValue = "%Ca+Hn/av";
+
+        final String encodedKey = "user%2Bname";
+        final String decodedKey = "user+name";
+
+        Map<String, String> newQuery = new HashMap<String, String>();
+        newQuery.put(decodedKey, decodedValue);
+
+        URI updated = PropertyUtil.replaceQuery(original, newQuery);
+
+        assertEquals(encodedKey + "=" + encodedValue, updated.getRawQuery());
+        assertEquals(decodedKey + "=" + decodedValue, updated.getQuery());
+    }
+
+    @Test
+    public void testReplaceQueryIgnoresQueryInCompositeURI() throws URISyntaxException {
+        URI original = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)");
+        URI expected = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)?failover.maxReconnectAttempts=1");
+        URI updated = PropertyUtil.replaceQuery(original, "failover.maxReconnectAttempts=1");
+
+        assertEquals(expected, updated);
+    }
+
+    @Test
+    public void testReplaceQueryIgnoresQueryAndFragmentInCompositeURI() throws URISyntaxException {
+        URI original = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true#ignored)");
+        URI expected = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true#ignored)?failover.maxReconnectAttempts=1");
+        URI updated = PropertyUtil.replaceQuery(original, "failover.maxReconnectAttempts=1");
+
+        assertEquals(expected, updated);
+    }
+
+    @Test
+    public void testReplaceQueryIgnoresQueryInCompositeURIPreservesFragment() throws URISyntaxException {
+        URI original = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)#fragment");
+        URI expected = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)?failover.maxReconnectAttempts=1#fragment");
+        URI updated = PropertyUtil.replaceQuery(original, "failover.maxReconnectAttempts=1");
+
+        assertEquals(expected, updated);
+    }
+
+    @Test
+    public void testReplaceQueryIgnoresQueryInCompositeURIReplaceExisting() throws URISyntaxException {
+        URI original = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)?failover.maxReconnectAttempts=2");
+        URI expected = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)?failover.maxReconnectAttempts=1");
+        URI updated = PropertyUtil.replaceQuery(original, "failover.maxReconnectAttempts=1");
+
+        assertEquals(expected, updated);
+    }
+
+    @Test
+    public void testReplaceQueryIgnoresQueryInCompositeURIReplaceExistingPreservesFragment() throws URISyntaxException {
+        URI original = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)?failover.maxReconnectAttempts=2#fragment");
+        URI expected = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)?failover.maxReconnectAttempts=1#fragment");
+        URI updated = PropertyUtil.replaceQuery(original, "failover.maxReconnectAttempts=1");
+
+        assertEquals(expected, updated);
+    }
+
+    //----- eraseQuery -------------------------------------------------------//
+
     @Test
     public void testEraseQuery() throws URISyntaxException {
         URI original = new URI("http://www.example.com?option=true");
@@ -195,6 +312,44 @@ public class PropertyUtilTest {
         assertNull(updated.getQuery());
     }
 
+    @Test
+    public void testEraseQueryPreservesFragment() throws URISyntaxException {
+        URI original = new URI("http://www.example.com?option=true#fragment");
+        URI updated = PropertyUtil.eraseQuery(original);
+
+        assertNull(updated.getQuery());
+        assertEquals("fragment", updated.getFragment());
+    }
+
+    @Test
+    public void testEraseQueryPreservesFragmentWhenNoQueryPresent() throws URISyntaxException {
+        URI original = new URI("http://www.example.com#fragment");
+        URI updated = PropertyUtil.eraseQuery(original);
+
+        assertNull(updated.getQuery());
+        assertEquals("fragment", updated.getFragment());
+    }
+
+    @Test
+    public void testEraseQueryIgnoresQueryInCompositeURI() throws URISyntaxException {
+        URI original = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)");
+        URI expected = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true)");
+        URI updated = PropertyUtil.eraseQuery(original);
+
+        assertEquals(expected, updated);
+    }
+
+    @Test
+    public void testEraseQueryIgnoresQueryAndFragmentInCompositeURI() throws URISyntaxException {
+        URI original = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true#ignored)?failover.maxReconnectAttempts=1");
+        URI expected = new URI("failover:(amqp://example.com:5672?amqp.traceFrames=true#ignored)");
+        URI updated = PropertyUtil.eraseQuery(original);
+
+        assertEquals(expected, updated);
+    }
+
+    //----- createQuery ------------------------------------------------------//
+
     @Test
     public void testCreateQuery() throws URISyntaxException {
         Map<String, String> newQuery = new HashMap<String, String>();
@@ -208,9 +363,28 @@ public class PropertyUtilTest {
     }
 
     @Test
-    public void testParseParametersFromURI() throws Exception {
+    public void testCreateQueryEncodesParameters() throws URISyntaxException {
+
+        final String encodedValue = "%25Ca%2BHn%2Fav";
+        final String decodedValue = "%Ca+Hn/av";
+
+        final String encodedKey = "user%2Bname";
+        final String decodedKey = "user+name";
+
+        Map<String, String> source = new HashMap<String, String>();
+        source.put(decodedKey, decodedValue);
+
+        String result = PropertyUtil.createQueryString(source);
+
+        assertEquals(encodedKey + "=" + encodedValue, result);
+    }
+
+    //----- parseQuery -------------------------------------------------------//
+
+    @Test
+    public void testParseQueryFromURI() throws Exception {
         URI original = new URI("http://www.example.com?option=true&another=false");
-        Map<String, String> result = PropertyUtil.parseParameters(original);
+        Map<String, String> result = PropertyUtil.parseQuery(original);
 
         assertEquals(2, result.size());
         assertTrue(result.containsKey("option"));
@@ -220,47 +394,22 @@ public class PropertyUtilTest {
     }
 
     @Test
-    public void testParseParametersFromURIWithNoQuery() throws Exception {
+    public void testParseQueryFromURIWithNoQuery() throws Exception {
         URI original = new URI("http://www.example.com");
-        Map<String, String> result = PropertyUtil.parseParameters(original);
+        Map<String, String> result = PropertyUtil.parseQuery(original);
         assertNotNull(result);
         assertTrue(result.isEmpty());
     }
 
     @Test
-    public void testParseParametersFromNullURI() throws Exception {
-        Map<String, String> result = PropertyUtil.parseParameters((URI) null);
+    public void testParseQueryFromNullURI() throws Exception {
+        Map<String, String> result = PropertyUtil.parseQuery((URI) null);
         assertNotNull(result);
         assertTrue(result.isEmpty());
     }
 
     @Test
-    public void testParseParametersFromString() throws Exception {
-        Map<String, String> result = PropertyUtil.parseParameters("http://www.example.com?option=true&another=false");
-
-        assertTrue(result.size() == 2);
-        assertTrue(result.containsKey("option"));
-        assertTrue(result.containsKey("another"));
-        assertEquals("true", result.get("option"));
-        assertEquals("false", result.get("another"));
-    }
-
-    @Test
-    public void testParseParametersFromURIStringWithNoQuery() throws Exception {
-        Map<String, String> result = PropertyUtil.parseParameters("http://www.example.com");
-        assertNotNull(result);
-        assertTrue(result.isEmpty());
-    }
-
-    @Test
-    public void testParseParametersFromNullURIString() throws Exception {
-        Map<String, String> result = PropertyUtil.parseParameters((String) null);
-        assertNotNull(result);
-        assertTrue(result.isEmpty());
-    }
-
-    @Test
-    public void testParseQuery() throws Exception {
+    public void testParseQueryFromString() throws Exception {
         Map<String, String> result = PropertyUtil.parseQuery("option=true&another=false");
 
         assertTrue(result.size() == 2);
@@ -270,6 +419,24 @@ public class PropertyUtilTest {
         assertEquals("false", result.get("another"));
     }
 
+    @Test
+    public void testParseQueryFromStringWithNoValues() throws Exception {
+        Map<String, String> result = PropertyUtil.parseQuery("option=&another=");
+
+        assertTrue(result.size() == 2);
+        assertTrue(result.containsKey("option"));
+        assertTrue(result.containsKey("another"));
+        assertEquals("", result.get("option"));
+        assertEquals("", result.get("another"));
+    }
+
+    @Test
+    public void testParseQueryFromNullURIString() throws Exception {
+        Map<String, String> result = PropertyUtil.parseQuery((String) null);
+        assertNotNull(result);
+        assertTrue(result.isEmpty());
+    }
+
     @Test
     public void testParseQueryWithSingletonProperty() throws Exception {
         Map<String, String> result = PropertyUtil.parseQuery("option=true&another=false&notAssigned");
@@ -297,6 +464,22 @@ public class PropertyUtilTest {
         assertTrue(result.isEmpty());
     }
 
+    @Test
+    public void testParseQueryDecodesParameters() throws Exception {
+        URI original = new URI("http://www.example.com?user%2Bname=%25Ca%2BHn%2Fav");
+
+        final String decodedKey = "user+name";
+        final String decodedValue = "%Ca+Hn/av";
+
+        Map<String, String> result = PropertyUtil.parseQuery(original);
+
+        assertEquals(1, result.size());
+        assertTrue(result.containsKey(decodedKey));
+        assertEquals(decodedValue, result.get(decodedKey));
+    }
+
+    //----- filterProperties -------------------------------------------------//
+
     @Test(expected=IllegalArgumentException.class)
     public void testFilterPropertiesNullProperties() throws Exception {
         PropertyUtil.filterProperties(null, "option.");
@@ -320,101 +503,7 @@ public class PropertyUtilTest {
         assertEquals("false", result.get("filtered2"));
     }
 
-    @Test
-    public void testAddPropertiesToURIFromBean() throws Exception {
-        String uriBase = "www.example.com";
-        Options configObject = new Options("foo", "bar");
-
-        String result = PropertyUtil.addPropertiesToURIFromBean(uriBase, configObject);
-        assertNotNull(result);
-        URI resultURI = new URI(result);
-        assertNotNull(resultURI.getQuery());
-
-        Map<String, String> props = PropertyUtil.parseQuery(resultURI.getQuery());
-        assertFalse(props.isEmpty());
-        assertTrue(props.containsKey("firstName"));
-        assertTrue(props.containsKey("lastName"));
-    }
-
-    @Test
-    public void testAddPropertiesToURIFromNullBean() throws Exception {
-        String uriBase = "www.example.com";
-        String result = PropertyUtil.addPropertiesToURIFromBean(uriBase, (Options) null);
-        assertNotNull(result);
-        URI resultURI = new URI(result);
-        assertNull(resultURI.getQuery());
-    }
-
-    @Test
-    public void testAddPropertiesToStringURIFromMap() throws Exception {
-        String uriBase = "www.example.com";
-
-        Map<String, String> properties = new HashMap<String, String>();
-        properties.put("option1", "true");
-        properties.put("option2", "false");
-
-        String result = PropertyUtil.addPropertiesToURI(uriBase, properties);
-        assertNotNull(result);
-        URI resultURI = new URI(result);
-        assertNotNull(resultURI.getQuery());
-
-        Map<String, String> parsed = PropertyUtil.parseQuery(resultURI.getQuery());
-        assertEquals(properties, parsed);
-    }
-
-    @Test
-    public void testAddPropertiesToStringURIFromEmptyMap() throws Exception {
-        String uriBase = "www.example.com";
-
-        String result = PropertyUtil.addPropertiesToURI(uriBase, null);
-        assertSame(uriBase, result);
-    }
-
-    @Test
-    public void testAddPropertiesToStringURIWithNullURI() throws Exception {
-        Map<String, String> properties = new HashMap<String, String>();
-        properties.put("option1", "true");
-        properties.put("option2", "false");
-
-        String result = PropertyUtil.addPropertiesToURI((String) null, properties);
-        assertNull(result);
-    }
-
-    @Test
-    public void testAddPropertiesToStringURIFromMapKeepsExisting() throws Exception {
-        String uriBase = "www.example.com?existing=keepMe";
-
-        Map<String, String> properties = new HashMap<String, String>();
-        properties.put("option1", "true");
-        properties.put("option2", "false");
-
-        String result = PropertyUtil.addPropertiesToURI(uriBase, properties);
-        assertNotNull(result);
-        URI resultURI = new URI(result);
-        assertNotNull(resultURI.getQuery());
-
-        Map<String, String> parsed = PropertyUtil.parseQuery(resultURI.getQuery());
-        assertEquals(3, parsed.size());
-        assertTrue(parsed.containsKey("existing"));
-        assertEquals("keepMe", parsed.get("existing"));
-    }
-
-    @Test
-    public void testAddPropertiesToURIFromMap() throws Exception {
-        URI uri = new URI("www.example.com");
-
-        Map<String, String> properties = new HashMap<String, String>();
-        properties.put("option1", "true");
-        properties.put("option2", "false");
-
-        String result = PropertyUtil.addPropertiesToURI(uri, properties);
-        assertNotNull(result);
-        URI resultURI = new URI(result);
-        assertNotNull(resultURI.getQuery());
-
-        Map<String, String> parsed = PropertyUtil.parseQuery(resultURI.getQuery());
-        assertEquals(properties, parsed);
-    }
+    //----- setProperties ----------------------------------------------------//
 
     @Test
     public void testSetProperties() throws Exception {
@@ -430,6 +519,15 @@ public class PropertyUtilTest {
         assertEquals("bar", configObject.getLastName());
     }
 
+    @Test(expected = IllegalArgumentException.class)
+    public void testSetPropertiesThrowsOnNullObject() throws Exception {
+        Map<String, String> properties = new HashMap<String, String>();
+        properties.put("firstName", "foo");
+        properties.put("lastName", "bar");
+
+        PropertyUtil.setProperties(null, properties);
+    }
+
     @Test
     public void testSetPropertiesUsingPropertiesObject() throws Exception {
         Options configObject = new Options();
@@ -480,6 +578,8 @@ public class PropertyUtilTest {
         assertEquals("bar", configObject.getLastName());
     }
 
+    //----- getProperties ----------------------------------------------------//
+
     @Test
     public void testGetProperties() throws Exception {
         Options configObject = new Options("foo", "bar");
@@ -556,6 +656,8 @@ public class PropertyUtilTest {
         assertNull(result);
     }
 
+    //----- setProperty ------------------------------------------------------//
+
     @Test
     public void testSetProperty() throws Exception {
         Options configObject = new Options();
@@ -640,6 +742,29 @@ public class PropertyUtilTest {
         PropertyUtil.setProperties(new Options(), (Properties) null);
     }
 
+    @Test
+    public void testSetPropertyWithTwoSettersUsingString() throws Exception {
+        Options configObject = new Options();
+        assertTrue(PropertyUtil.setProperty(configObject, "intOption", "1"));
+        assertEquals(1, configObject.getIntOption());
+    }
+
+    @Test
+    public void testSetPropertyWithTwoSettersUsingInt() throws Exception {
+        Options configObject = new Options();
+        assertTrue(PropertyUtil.setProperty(configObject, "intOption", 1));
+        assertEquals(1, configObject.getIntOption());
+    }
+
+    @Test
+    public void testSetPropertyWithTwoSettersUsingBadValueIsNotApplied() throws Exception {
+        Options configObject = new Options();
+        assertFalse(PropertyUtil.setProperty(configObject, "intOption", Long.MAX_VALUE));
+        assertEquals(0, configObject.getIntOption());
+    }
+
+    //----- stripPrefix ------------------------------------------------------//
+
     @Test
     public void testStripPrefix() {
         String value = "prefixed.option";
@@ -659,6 +784,8 @@ public class PropertyUtilTest {
         assertNull(PropertyUtil.stripPrefix((String) null, "prefixed."));
     }
 
+    //----- stripBefore ------------------------------------------------------//
+
     @Test
     public void testStripBefore() {
         String value = "prefixed.option";
@@ -671,6 +798,8 @@ public class PropertyUtilTest {
         assertNull(PropertyUtil.stripBefore((String) null, '.'));
     }
 
+    //----- stripUpto ------------------------------------------------------//
+
     @Test
     public void testStripUpTo() {
         String value = "prefixed.option";
@@ -682,4 +811,5 @@ public class PropertyUtilTest {
     public void testStripUpToNullString() {
         assertNull(PropertyUtil.stripUpto((String) null, '.'));
     }
+
 }
