@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -72,7 +72,7 @@ public class JmsTxConsumerFailoverTest extends AmqpTestSupport {
         assertTrue("Should have a new connection.", Wait.waitFor(new Wait.Condition() {
 
             @Override
-            public boolean isSatisified() throws Exception {
+            public boolean isSatisfied() throws Exception {
                 return brokerService.getAdminView().getCurrentConnectionsCount() == 1;
             }
         }, TimeUnit.SECONDS.toMillis(30), TimeUnit.MILLISECONDS.toMillis(100)));
@@ -80,7 +80,7 @@ public class JmsTxConsumerFailoverTest extends AmqpTestSupport {
         assertTrue("Should have a recovered consumer.", Wait.waitFor(new Wait.Condition() {
 
             @Override
-            public boolean isSatisified() throws Exception {
+            public boolean isSatisfied() throws Exception {
                 return brokerService.getAdminView().getQueueSubscribers().length == 1;
             }
         }, TimeUnit.SECONDS.toMillis(30), TimeUnit.MILLISECONDS.toMillis(50)));
@@ -128,6 +128,7 @@ public class JmsTxConsumerFailoverTest extends AmqpTestSupport {
         assertEquals(MSG_COUNT, proxy.getQueueSize());
 
         try {
+            LOG.info("Session commit firing after connection failed.");
             session.commit();
             fail("Session commit should have failed with TX rolled back.");
         } catch (TransactionRolledBackException rb) {
@@ -168,9 +169,11 @@ public class JmsTxConsumerFailoverTest extends AmqpTestSupport {
         assertEquals(MSG_COUNT, proxy.getQueueSize());
 
         try {
+            LOG.info("Transacted being rolled back after failover");
             session.rollback();
-            LOG.info("Transacted rollback after failover ok");
+            LOG.info("Transacted rollback after failover");
         } catch (JMSException ex) {
+            LOG.info("Caught unexpected error: {}", ex.getMessage());
             fail("Session rollback should not have failed.");
         }
 
